from charmhelpers.core import templating
from charmhelpers.core import host


SERVICES = {}


def register(services, templates_dir=None):
    """
    Register a list of service configs.

    Service Configs are dicts in the following formats:

        {
            "service": <service name>,
            "templates": [ {
                'target': <render target of template>,
                'source': <optional name of template in passed in templates_dir>
                'file_properties': <optional dict taking owner and octal mode>
                'contexts': [ context generators, see contexts.py ]
                }
            ] }

    Either `source` or `target` must be provided.

    If 'source' is not provided for a template the templates_dir will
    be consulted for ``basename(target).j2``.

    If `target` is not provided, it will be assumed to be
    ``/etc/init/<service name>.conf``.
    """
    for service in services:
        service.setdefault('templates_dir', templates_dir)
        SERVICES[service['service']] = service


def reconfigure_services(restart=True):
    """
    Update all files for all services and optionally restart them, if ready.
    """
    for service_name in SERVICES.keys():
        reconfigure_service(service_name, restart=restart)


def reconfigure_service(service_name, restart=True):
    """
    Update all files for a single service and optionally restart it, if ready.
    """
    service = SERVICES.get(service_name)
    if not service or service['service'] != service_name:
        raise KeyError('Service not registered: %s' % service_name)

    manager_type = service.get('type', UpstartService)
    manager_type(service).reconfigure(restart)


def stop_services():
    for service_name in SERVICES.keys():
        if host.service_running(service_name):
            host.service_stop(service_name)


class ServiceTypeManager(object):
    def __init__(self, service_definition):
        self.service_name = service_definition['service']
        self.templates = service_definition['templates']
        self.templates_dir = service_definition['templates_dir']

    def reconfigure(self, restart=True):
        raise NotImplementedError()


class UpstartService(ServiceTypeManager):
    def __init__(self, service_definition):
        super(UpstartService, self).__init__(service_definition)
        for tmpl in self.templates:
            if 'target' not in tmpl:
                tmpl['target'] = '/etc/init/%s.conf' % self.service_name

    def reconfigure(self, restart):
        complete = templating.render(self.templates, self.templates_dir)

        if restart and complete:
            host.service_restart(self.service_name)
